from datetime import datetime, timedelta

from tc_analyzer_lib.metrics.neo4j_analytics import Neo4JAnalytics
from tc_analyzer_lib.schemas import GraphSchema
from tc_neo4j_lib.neo4j_ops import Neo4jOps


def test_avg_clustering_exclude_past():
    """
    test scaling of the avgClusteringCoefficient (a.k.a fragmentation score)
    """
    neo4j_ops = Neo4jOps.get_instance()

    # deleting all data
    neo4j_ops.gds.run_cypher("MATCH (n) DETACH DELETE (n)")

    # timestamps
    today = 1689280200.0
    yesterday = 1689193800.0
    past_three_days = (
        datetime.fromtimestamp(yesterday) - timedelta(days=2)
    ).timestamp()
    past_window_date = (
        datetime.fromtimestamp(yesterday) - timedelta(days=1)
    ).timestamp()
    graph_schema = GraphSchema(platform="discord")
    platform_id = "5151515151515"
    neo4j_analytics = Neo4JAnalytics(platform_id, graph_schema)

    user_label = graph_schema.user_label
    platform_label = graph_schema.platform_label
    interacted_with = graph_schema.interacted_with_rel
    is_member = graph_schema.member_relation

    # creating some nodes with data
    neo4j_ops.gds.run_cypher(
        f"""
        CREATE (a:{user_label}) -[:{is_member}]->(g:{platform_label} {{guildId: '{platform_id}'}})
        CREATE (b:{user_label}) -[:{is_member}]->(g)
        CREATE (c:{user_label}) -[:{is_member}]->(g)
        CREATE (d:{user_label}) -[:{is_member}]->(g)
        CREATE (e:{user_label}) -[:{is_member}]->(g)
        SET a.id = "1000"
        SET b.id = "1001"
        SET c.id = "1002"
        SET d.id = "1003"
        SET e.id = "1004"
        MERGE (a) -[r:{interacted_with} {{date: {yesterday}, weight: 1}}]->(b)
        MERGE (a) -[r2:{interacted_with} {{date: {today}, weight: 2}}]->(b)
        MERGE (a) -[r3:{interacted_with} {{date: {yesterday}, weight: 3}}]->(d)
        MERGE (c) -[r4:{interacted_with} {{date: {yesterday}, weight: 2}}]->(b)
        MERGE (c) -[r5:{interacted_with} {{date: {today}, weight: 1}}]->(b)
        MERGE (c) -[r6:{interacted_with} {{date: {yesterday}, weight: 2}}]->(d)
        MERGE (d) -[r7:{interacted_with} {{date: {yesterday}, weight: 1}}]->(b)
        MERGE (c) -[r8:{interacted_with} {{date: {today}, weight: 2}}]->(a)
        MERGE (d) -[r9:{interacted_with} {{date: {today}, weight: 1}}]->(c)
        MERGE (b) -[r10:{interacted_with} {{date: {today}, weight: 2}}]->(d)
        MERGE (d) -[r11:{interacted_with} {{date: {today}, weight: 1}}]->(c)
        MERGE (e) -[r12:{interacted_with} {{date: {today}, weight: 3}}]->(b)
        MERGE (a) -[r13:{interacted_with} {{date: {past_three_days}, weight: 3}}]->(d)
        MERGE (d) -[r14:{interacted_with} {{date: {past_three_days}, weight: 3}}]->(b)
        MERGE (b) -[r15:{interacted_with} {{date: {past_three_days}, weight: 3}}]->(e)
        MERGE (e) -[r16:{interacted_with} {{date: {past_three_days}, weight: 3}}]->(c)

        SET r.platformId = '{platform_id}'
        SET r2.platformId = '{platform_id}'
        SET r3.platformId = '{platform_id}'
        SET r4.platformId = '{platform_id}'
        SET r5.platformId = '{platform_id}'
        SET r6.platformId = '{platform_id}'
        SET r7.platformId = '{platform_id}'
        SET r8.platformId = '{platform_id}'
        SET r9.platformId = '{platform_id}'
        SET r10.platformId = '{platform_id}'
        SET r11.platformId = '{platform_id}'
        SET r12.platformId = '{platform_id}'
        SET r13.platformId = '{platform_id}'
        SET r14.platformId = '{platform_id}'
        SET r15.platformId = '{platform_id}'
        SET r16.platformId = '{platform_id}'
        """
    )

    neo4j_analytics.compute_local_clustering_coefficient(from_start=False)
    fragmentation_score = neo4j_analytics.compute_fragmentation_score(
        past_window_date=past_window_date,
        scale_fragmentation_score=200,
    )

    for score in fragmentation_score:
        if score["date"] == yesterday:
            assert score["fragmentation_score"] - 166.6666 < 0.1
        elif score["date"] == today:
            assert score["fragmentation_score"] - 120 < 0.1
        else:
            print(score["date"])
            # is shouldn't ever get here since
            # we do compute for just two periods
            assert False is True
